@@ -1,5 +1,5 @@
 package B::Stats;
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
 =head1 NAME
 
@@ -25,6 +25,8 @@ The purpose is to help you in your goal:
 
 =head1 OPTIONS
 
+Options can be bundled: C<-c,-e,-r> eq C<-cer>
+
 =over
 
 =item -c I<static>
@@ -44,7 +46,7 @@ Single ops can be called multiple times.
 
 =item -a I<all (default)>
 
-Same as -c,-e,-r: static compile-time, end-time and dynamic run-time.
+Same as -cer: static compile-time, end-time and dynamic run-time.
 
 =item -t I<table>
 
@@ -70,12 +72,14 @@ to the op.
 
 Filter for op names and classes. Only calculate the given ops, resp. op class.
 
-  perl -MB::Stats,-fLOGOP,COP,concat myprog.pl
+  perl -MB::Stats,-fLOGOP,-fCOP,-fconcat myprog.pl
 
-=item -lF<logfile> B<NOT YET>
+=item -lF<logfile>
 
 Print output only to this file. Default: STDERR
 
+  perl -MB::Stats,-llog myprog.pl
+
 =back
 
 =head1 METHODS
@@ -96,7 +100,7 @@ use B::Stats::Minus;
 # Opcodes-0.10 adds 6 files and 5303-3821 lines: Carp, AutoLoader, subs
 # Opcodes-0.11 adds 2 files and 4141-3821 lines: subs
 # use Opcodes; # deferred to run-time below
-our ($static, @runtime, $compiled);
+our ($static, @runtime, $compiled, $imported, $LOG);
 my (%opt, $nops, $rops, @all_subs, $frag, %roots);
 my ($c_count, $e_count, $r_count);
 
@@ -115,11 +119,41 @@ sub import {
 	}
       } while $rest;
     }
+    # taking multiple arguments: -ffilter
+    if (/^-?f(.*)$/) {
+      my $arg = $1;
+      if ($arg =~ /^[A-Z]*OP$/) {
+	my @optype = qw(OP UNOP BINOP LOGOP LISTOP PMOP SVOP PADOP PVOP LOOP COP);
+	$opt{f}->{class}->{$arg} = 1;
+	die "invalid B::Stats,-fOPCLASS argument: $arg\n"
+	  unless grep {$arg eq $_} @optype;
+	# pre-expand names for the class
+	require Opcodes;
+	my $maxo = Opcodes::opcodes();
+	for my $i (0..$maxo-1) {
+	  my $name = Opcodes::opname($i);
+	  my $class = $optype[ Opcodes::opclass($i) ];
+	  if ($class eq $arg) {
+	    $opt{f}->{name}->{$name} = 1;
+	  }
+	}
+      } elsif ($arg =~ /^[a-z_]+$/) {
+	$opt{f}->{name}->{$arg} = 1;
+      } else {
+	die "invalid B::Stats,-ffilter argument: $arg\n";
+      }
+    }
+    if (/^-?(l)(.*)$/) { # taking arguments: -llogfile
+      $opt{$1} = $2;
+      open $LOG, ">", $opt{l} or die "Cannot write to B::Stats,-llogfile: $opt{l}\n";
+    }
   }
-  # -ffilter and -llog not yet
+
   $opt{a} = 1 if !$opt{c} and !$opt{e} and !$opt{r}; # default
   $opt{c} = $opt{e} = $opt{r} = 1 if $opt{a};
 #warn "%opt: ",keys %opt,"\n"; # for Debugging
+  $LOG = \*STDERR unless $opt{l};
+  $imported = 1;
 }
 
 sub _class {
@@ -223,7 +257,7 @@ Static -c check at CHECK time. Triggered by -MO=Stats,-OPTS
 =cut
 
 sub compile {
-  import(@_); # check options via O
+  import(@_) unless $imported; # check options via O
   $compiled++;
   $opt{c} = 1;
   return sub {
@@ -246,40 +280,43 @@ General formatter
 sub output {
   my ($count, $ops, $name) = @_;
 
-  my $files = scalar keys %B_inc;
-  my $lines = 0;
-  for (values %B_inc) {
-    print STDERR $_,"\n" if $opt{F};
-    open IN, "<", "$_";
-    # Todo: skip pod?
-    while (<IN>) { chomp; s/#.*//; next if not length $_; $lines++; };
-    close IN;
-  }
   my %name = (
     'static compile-time' => 'c',
     'static end-time'     => 'e',
     'dynamic run-time'    => 'r'
     );
   my $key = $name{$name};
+  my $lines = 0;
+  my $inc = $key eq 'c' ? \%B_inc : \%INC;
+  my $files = scalar keys %$inc;
+  for (values %$inc) {
+    print $LOG $_,"\n" if $opt{F};
+    open IN, "<", "$_";
+    # Todo: skip pod?
+    while (<IN>) { chomp; s/#.*//; next if not length $_; $lines++; };
+    close IN;
+  }
   $files -= $B::Stats::Minus::overhead{$key}{_files};
   $lines -= $B::Stats::Minus::overhead{$key}{_lines};
   $ops -= $B::Stats::Minus::overhead{$key}{_ops};
-  print STDERR "\nB::Stats $name:\nfiles=$files\tlines=$lines\tops=$ops\n";
+  print $LOG "\nB::Stats $name:\nfiles=$files\tlines=$lines\tops=$ops\n";
   return if $opt{t} and $opt{u};
 
-  print STDERR "\nop name:\n";
+  print $LOG "\nop name:\n";
   for (sort { $count->{name}->{$b} <=> $count->{name}->{$a} }
        keys %{$count->{name}}) {
     my $l = length $_;
     my $c = $count->{name}->{$_} - $B::Stats::Minus::overhead{$key}{$_};
-    print STDERR $_, " " x (10-$l), "\t", $c, "\n";
+    next if $opt{f} and !$opt{f}->{name}->{$_};
+    print $LOG $_, " " x (10-$l), "\t", $c, "\n";
   }
   unless ($opt{u}) {
-    print STDERR "\nop class:\n";
+    print $LOG "\nop class:\n";
     for (sort { $count->{class}->{$b} <=> $count->{class}->{$a} }
 	 keys %{$count->{class}}) {
+      next if $opt{f} and !$opt{f}->{class}->{$_};
       my $l = length $_;
-      print STDERR $_, " " x (10-$l), "\t", $count->{class}->{$_}, "\n";
+      print $LOG $_, " " x (10-$l), "\t", $count->{class}->{$_}, "\n";
     }
   }
 }
@@ -308,9 +345,11 @@ sub output_runtime {
       my $name = Opcodes::opname($i);
       if ($name) {
 	my $class = $optype[ Opcodes::opclass($i) ];
+	next if $opt{f} and !$opt{f}->{name}->{$name};
+	next if $opt{f} and !$opt{f}->{class}->{$class};
 	$r_count->{name}->{ $name } += $count;
-	$r_count->{class}->{ $class } += $count;
 	$rops += $count;
+	$r_count->{class}->{ $class } += $count;
       } else {
 	warn "invalid name for opcount[$i]";
       }
@@ -325,28 +364,39 @@ sub output_runtime {
 
 =cut
 
+sub _output_tline {
+  my $n = shift;
+  my $name = $n.(" "x(12-length($n)));
+  return if $opt{f} and !$opt{f}->{name}->{$n};
+  print $LOG join("\t",
+		    ($name,
+		     $c_count->{name}->{$n},
+		     $e_count->{name}->{$n},
+		     $r_count->{name}->{$n})),
+               "\n";
+}
+
 sub output_table {
   my ($c, $e, $r) = @_;
-  format STDERR_TOP =
-
+  # XXX we have empty runops runs with format.
+  #my $x = 0;
+  #for (keys %{$r_count->{name}}) {
+  #  $x++ if $r_count->{name}->{$_};
+  #}
+  #return unless $x;
+  print "
 B::Stats table:
-@<<<<<<<<<<	@>>>>	@>>>>	@>>>>
-"",             "-c",   "-e",   "-r"
-.
-  write STDERR;
-  format STDERR =
-@<<<<<<<<<<	@>>>>	@>>>>	@>>>>
-$_,$c_count->{name}->{$_},$e_count->{name}->{$_},$r_count->{name}->{$_}
-.
+           	-c	-e	-r
+";
   if (%$e_count) {
     for (sort { $e_count->{name}->{$b} <=> $e_count->{name}->{$a} }
          keys %{$e_count->{name}}) {
-      write STDERR;
+      _output_tline($_);
     }
   } else {
     for (sort { $c_count->{name}->{$b} <=> $c_count->{name}->{$a} }
          keys %{$c_count->{name}}) {
-      write STDERR;
+      _output_tline($_);
     }
   }
 }
@@ -370,7 +420,9 @@ END {
     $e_count = $static;
   }
   output_runtime() if $opt{r};
-  output_table($c_count, $e_count, $r_count) if $opt{t};
+  if ( $opt{t} and ($] < 5.014 or ${^GLOBAL_PHASE} ne 'DESTRUCT') ) {
+    output_table($c_count, $e_count, $r_count);
+  }
 }
 
 1;
